from  xml.sax import parse
from xml.sax.handler import ContentHandler
import os


class Dispatcher:
    def dispatch(self, prefix, name, attrs=None):  # 定义调用某一具体方法的函数
        mname = prefix + name.capitalize()  # 具体方法名为prefix（前缀）连接首字母大写的name
        dname = 'default' + prefix.capitalize()  # 默认方法名为default连接首字母大写的prefix
        method = getattr(self, mname, None)  # 通过具体方法名获取方法对象
        if callable(method):  # 如果具体方法可调用
            args = ()  # 创建空的参数元组
        else:
            method = getattr(self, dname, None)  # 通过默认方法获取默认方法对象
            args = (name,)  # 创建参数为元组
        if prefix == 'start':  # 如果是开始元素
            args += (attrs,)  # 合并参数元组为新参数元组
        if callable(method):  # 如果方法可调用
            method(*args)  # 传入参数调用方法

    def startElement(self, name, attrs):  # 重写开始元素的方法
        self.dispatch('start', name, attrs)  # 通过函数dispatch()调用某一具体方法

    def endElement(self, name):  # 重写结束元素的方法
        self.dispatch('end', name)  # 通过函数dispatch()调用某一具体方法


class XMLHandler(Dispatcher, ContentHandler):
    def __init__(self, directory):
        super().__init__()
        self.in_page = False  # 创建开关变量
        self.directory = [directory]  # 创建目录列表
        self.ensureDirectory()  # 创建根目录

    def ensureDirectory(self):  # 定义创建目录的方法
        path = os.path.join(*self.directory)  # 定义路径
        if not os.path.isdir(path):  # 如果路径指向的目录不存在
            os.makedirs(path)  # 创建目录

    def startDirectory(self, attrs):  # 定义读取到目录开始标记的方法
        self.directory.append(attrs['name'])  # 添加目录名称到目录列表
        self.ensureDirectory()  # 创建目录

    def endDirectory(self):  # 定义读取到目录结束标记的方法
        self.directory.pop()  # 弹出目录

    def writeHeader(self, title):  # 定义写入头部HTML代码的方法
        self.file.write(
            '<html>\n<head>\n<meta charset="gbk">\n<title>{}</title>\n</head>\n<body>\n'.format(title))

    def writeFooter(self, ):  # 定义写入脚部HTML代码的方法
        self.file.write('\n</body>\n</html>\n')

    def startPage(self, attrs):  # 定义读取到页面开始标记的方法
        filename = os.path.join(*self.directory + [attrs['name'] + '.html'])  # 创建页面路径
        self.in_page = True  # 打开写入页面内容的开关
        self.file = open(filename, 'w')  # 创建HTML页面文件
        self.writeHeader(attrs['title'])  # 写入页面头部的HTML代码

    def endPage(self):  # 定义读取到页面结束标记的方法
        self.in_page = False  # 关闭写入页面内容的开关
        self.writeFooter()  # 写入页面脚部的HTML代码
        self.file.close()  # 关闭创建的HTML文件

    def defaultStart(self, name, attrs):  # 定义读取到开始标记的默认方法
        if self.in_page:  # 如果写入页面内容的开关被打开
            self.file.write('<' + name)  # 写入开始标记
            for key, value in attrs.items():  # 循环读取属性的键值
                self.file.write(' {}="{}"'.format(key, value))  # 写入开始标记的属性
            self.file.write('>')  # 写入开始标记的结束符号

    def defaultEnd(self, name):  # 定义读取到结束标记的默认方法
        if self.in_page:  # 如果写入页面内容的开关被打开
            self.file.write('</' + name + '>')  # 写入结束标记

    def characters(self, content):  # 定义读取到标记内容的方法
        if self.in_page:  # 如果写入页面内容的开关被打开
            self.file.write(content)  # 写入标记中的内容


if __name__ == '__main__':
    parse('website.xml', XMLHandler('html'))  # 调用解析函数
